import java.util.*;
import java.lang.*;
import java.lang.Math;
import java.lang.Integer;


/**
 * Title:        Kmeans.java
 * Description:  Class for  kmeans-algorithm.
 * Copyright:    Annemari Auvinen, Anu Niemi (c) 2002
 * Company:
 * @author       Annemari Auvinen, Anu Niemi
 * @version      1.0
 *
 * @param   data        Vector for datapoints
 * @param   newW        Vector for midpoints.
 * @param   w           Vector for the values of midpoints.
 * @param   k           Int number of clusters
 * @param   epsilon     Double value of epsilon.
 * @param   classedData Vector for classified data.
 */

public class Kmeans {

  private Vector data = new Vector();
  private Vector w = new Vector();
  private Vector newW = new Vector();
  private Vector classedData = new Vector();
  private int k;
  private double epsilon = 0.1;

/**
  * Returns vector, which includes the midpoints sorted by descending order based on
  * value of x.
  * @return       Vector of midpoints
  */
  public Vector getW()
  {
    ArrangeVector arranged = new ArrangeVector();
    Vector classes = arranged.arrange(w);
    return classes;
  }

/**
  * Sets the epsilon.
  * @param    epsilon    Int value of epsilon
  */
  public void setEpsilon(double value)
  {
    epsilon = value;
  }
/**
  * Sets the number of clusters.
  * @param    value    Int value of clusters
  */
  public void setK(int value)
  {
    k = value;
  }

/**
  * Sets the w vector.
  */
  public void setWVector(){
    int points = data.size();
    int ii;
    Vector used = new Vector();
    Random random = new Random();
    Integer index = new Integer(0);

    for (int i = 0; i<k; i++)
    {
      do{
        ii = random.nextInt(points);
        index = new Integer(ii);
      }while(used.contains(index));

      used.add(index);
      w.add(data.elementAt(ii));
    }
   }

/**
  * Creates the datavectors where data will be classed.
  */
  public void createDataVectors()
  {
    for (int i = 0; i<k; i++)
    {
      classedData.add(new Vector());
    }
  }

/**
  * Constructor for Kmeans.
  * @param data        Vector of datapoints
  * @param k           Int number of clusters
  */
  public Kmeans(Vector data, int k ) {
    this.data = data;
    this.k = k;
    setWVector();
    createDataVectors();
    classData();
    do {
      avarages();
      boolean end =testEnd();
      if (end)
      {
         break;
      }
      else {
        w.removeAllElements();
        for (int i=0; i<k; i++)
        {
          Observation h = (Observation)newW.elementAt(i);
          w.add(h);
        }
        newW.removeAllElements();
        classedData.removeAllElements();
        createDataVectors();
        classData();
      }
    } while (true);

  }

/**
  * Classifies the data
  */
  public void classData()
  {
    double x;
    double y;

    double wx;
    double wy;

    double dist;  //distance
    int index = 0;

    for (int i=0; i<data.size(); i++)
    {
      dist = -1;
      Observation dataPoint = (Observation)data.elementAt(i);
      x = dataPoint.getX();
      y = dataPoint.getY();

      for (int j=0; j<k; j++)
      {
        Observation ww = (Observation)w.elementAt(j);
        wx = ww.getX();
        wy = ww.getY();
        double testDist;
        testDist = Math.sqrt( (Math.pow( (x-wx), 2.0) + Math.pow( (y-wy), 2.0) ) );

        if (dist < 0)
          {
            dist = testDist;
            index = j;
          }

        if (dist > testDist)
        {
          dist =testDist;
          index = j;
        }
      }

      Vector vector = (Vector)classedData.elementAt(index);
      vector.add(dataPoint);
    }

  }

/**
  * Calculates the avarages. Avarages are stored in the vector newW.
  */
  public void avarages()
  {
     double x = 0.0;
     double y = 0.0;

     for(int i=0; i<k; i++)
     {
       x = 0.0;
       y = 0.0;
       Vector classedPoints = (Vector)classedData.elementAt(i);
       int dataPoints = classedPoints.size();

       //X-value
       for (int j=0; j<dataPoints; j++)
       {
         Observation point = (Observation)classedPoints.elementAt(j);
         x = x + point.getX();
       }
       x = x / dataPoints;

       //y-value
       for (int j=0; j<dataPoints; j++)
       {
         Observation point = (Observation)classedPoints.elementAt(j);
         y = y + point.getY();
       }
       y = y / dataPoints;
      Observation o = new Observation(x,y);
      newW.add(o);
     }
  }


/**
  * Tests whether there is enoough iterations.
  *@return   true if the distance between the elements of w and newW vectors is
  *          less than epsilon, otherwise false
  */
  public boolean testEnd()
  {
    for(int i = 0; i<k; i++ )
    {
       Observation newWvalue = (Observation)newW.elementAt(i);
       Observation oldWvalue = (Observation)w.elementAt(i);
       double testDist;
       testDist = Math.sqrt( (Math.pow( (oldWvalue.getX()-newWvalue.getX()), 2.0) + Math.pow( (oldWvalue.getY()- newWvalue.getY()), 2.0) ) );
       if (testDist > epsilon )
       {
         return false;
       }
    }
    return true;
  }

}